############################
# section 2: Creating Java Bean 
############################
setGeneric("getExtPart", function(typeMode) standardGeneric("getExtPart"))
setMethod("getExtPart", signature(typeMode="robject"), 
    function(typeMode) paste(" extends ", primaryDataPkg(), ".RObject ", sep="", collapse="")
)

setMethod("getExtPart", signature(typeMode="javalib"), 
    function(typeMode) " implements java.io.Serializable "
)

setGeneric("getBaseObj", function(typeMode) standardGeneric("getBaseObj"))
setMethod("getBaseObj", signature(typeMode="robject"), 
    function(typeMode) paste(primaryDataPkg(), ".RObject", sep="", collapse="")
)

setMethod("getBaseObj", signature(typeMode="javalib"), 
    function(typeMode) "Object"
)
                            
getConstructorPart <- function(jType) {
    abstractIndex <- grep("Factory$", jType)
    rvectorIndex <- grep(paste(primaryDataPkg(), "RVector", sep="."), jType, fixed=T)
    abstractIndex <- c(abstractIndex, rvectorIndex)
    arrayIndex <- grep("\\[\\]$", jType)
    res <- paste("new ", jType, "()", sep="")
    res[abstractIndex] <- "null"
    res[arrayIndex] <- paste("new ", jType[arrayIndex], "{}", sep="")
    res
}

getToStringMethod <- function(jType, slotName) { 
    arrayIndex <- grep("\\[\\]$", jType) 
    res <- paste("String.valueOf(", slotName, ")", sep=" ")  
    res[arrayIndex] <- paste("java.util.Arrays.toString(", slotName[arrayIndex], ")", sep=" ")
    res
}

getEqualsMethod <- function(jType, slotName) {
    arrayIndex <- grep("\\[\\]$", jType)
    res <- paste(slotName, ".equals(input", capName(slotName), ")", sep="")
    res[arrayIndex] <- paste("java.util.Arrays.equals(", slotName[arrayIndex], ", ",
                        "(",jType[arrayIndex], ")input", capName(slotName[arrayIndex]), ")", sep="")
    res
}

## create the java bean corresponding to a R S4 class
## param:
##      rType: R S4 class name 
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
##      slotType: a named vector where names are slot names of the R S4 class, 
##                  and values are the corresponding java data types for the slots
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: definition of the java bean in character string
createJavaBean <- function( javaType, pkgName, slotType, classDoc, typeMode ) {
    beanBody <- paste( "package ", pkgName, ";\n",
                      "\n\t/**\n",
                      "\t* This file was auto-generated by R function \n",
                      "\t* createJavaBean ", date(), ". \n",
                      "\t* ", get("ClassDesc_RWebServices", classDoc), "\n",
                      "\t*/\n", 
                       sep="", collapse="")
    rSlotName <- names(slotType)
    slotName <- mangleName(rSlotName)
    beanBody <- paste(beanBody, "\n\npublic class ", javaType, getExtPart(typeMode),
                      " {\n", sep="", collapse="")
    fieldDef <- paste("\tprivate ", slotType, " ", slotName, ";\n",
                      sep="", collapse="")
    paramStr <- paste("\t\t", slotType, " ", slotName, sep="",
                      collapse=",\n")
    paramAssignStr <- paste("\t\tthis.", slotName, " = ", 
                            slotName, ";\n", sep="", collapse="")
    defaultAssignStr <- paste("\t\tthis.", slotName, " = ",
                            getConstructorPart(slotType), ";\n", sep="", collapse="")
    constructor <- paste("\n\tpublic ", javaType, "() {\n",
                         defaultAssignStr,
                         "\t}\n",
                         "\n\tpublic ", javaType, "(\n",
                         paramStr, "){\n\n",
                         paramAssignStr,
                         "\t}\n", sep="", collapse="")
    beanBody <- paste(beanBody, fieldDef, constructor, sep="", collapse="")
    setMethods <- paste("\n\t/**\n", 
                        "\t* Sets the ", slotName, " value for this ", javaType, "\n",
                        "\t*\n",
                        "\t* @param\t", slotName, "\t", get(rSlotName, classDoc), "\n",
                        "\t*/\n",
                        "\tpublic void set", capName(slotName), "(", slotType,
                        " ", slotName, ") {\n",
                        "\t\tthis.", slotName, " = ", slotName, ";\n",
                        "\t}\n", sep="")
    getMethods <- paste("\n\t/**\n", 
                        "\t* Gets the ", slotName, " value for this ", javaType, "\n",
                        "\t*\n",
                        "\t* @return\tthe value of ", slotName, "\n",
                        "\t*\t", get(rSlotName, classDoc), "\n",
                        "\t*/\n",
                        "\tpublic ", slotType, " get", capName(slotName), "() {\n",
                        "\t\treturn ", slotName, ";\n",
                        "\t}\n", sep="")
    methodPart <- paste(setMethods, getMethods, sep="", collapse="")
    
    printElement <- paste(
                        "\t\tres.append(\"", slotName, "=\" + ", 
                        getToStringMethod(slotType, slotName), ");\n", 
                        sep="", collapse="\t\tres.append(\", \");\n")                            
    printPart <- paste("\n\t/**\n", 
                       "\t* Print this ", javaType, " to String\n",
                       "\t*\n",
                       "\t* @return\tcontents of ", javaType, " as a String\n",
                       "\t*/\n",
                       "\tpublic String toString() {\n",
                       "\t\tStringBuffer res=new StringBuffer(\"{\");\n",
                       printElement,
                       "\t\tres.append(\"}\");\n",
                       "\t\treturn res.toString();\n",
                       "\t}\n", sep="", collapse="")
    equalsElement <- paste(
                        "\t\t\tObject input", capName(slotName), " = obj.get",
                        capName(slotName), "();\n",
                        "\t\t\tif((", slotName, "==null)||(input", capName(slotName), 
                        "==null))\n",
                        "\t\t\t\tres = res&&(", 
                        slotName, "==input", capName(slotName), ");\n",
                        "\t\t\telse\n",
                        "\t\t\t\tres = res&&", getEqualsMethod(slotType, slotName),
                        ";\n", sep="", collapse="") 
    equalsPart <- paste("\n\t/**\n", 
                       "\t* Compare this ", javaType, " with the input data.\n",
                       "\t*\n",
                       "\t* @param inputObject\tthe data to be compared with.\n",
                       "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
                       "\t*/\n",
                       "\tpublic boolean equals(Object inputObject) {\n",
                       "\t\tboolean res = getClass().equals(inputObject.getClass());\n",
                       "\t\tif(res){\n",
                       "\t\t\t", javaType, " obj = (", javaType, ")inputObject;\n",
                       equalsElement,
                       "\t\t}\n",
                       "\t\treturn res;\n",
                       "\t}\n",
                        sep="", collapse="")
    beanBody <- paste(beanBody, methodPart, printPart, equalsPart, "\n}\n", sep="", collapse="")
    beanBody
}

## create the java classes corresponding to a R ClassUnion
## param:
##      javaType: main java factory class name (short name, without package name)
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
##      subklass: a named vector where names are member class names of the R ClassUnion, 
##                  and values are the corresponding java data types for them
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: a named character vector: name is java class name (short name, 
##          without package), value is the definition of the java classes 
createUnionFactory <- function( javaType, pkgName, subklass, typeMode ) {
    header <- paste(  "*\n* This file was auto-generated from R function\n",
                      "* by R function crateUnionFactory ", date(), "\n",
                      "*/\n\n", 
                      "package ", pkgName, ";\n", sep="", collapse="")
    unionFactoryBody <- paste("\n/**\n* ", javaType, ".java\n", header,
        "\n\npublic abstract class ", javaType, getExtPart(typeMode), " {\n",
        "\n\t/**\n", 
        "\t* Sets the data value for this ", javaType, "\n",
        "\t*\n",
        "\t* @param\tdata\tdata should be an instance of ", 
        paste(subklass, sep="", collapse=" or "), "\n",
        "\t*/\n", 
        "\tpublic abstract void setData( ", getBaseObj(typeMode), " data ) throws Exception;\n", 
        "\n\t/**\n", 
        "\t* Gets the data value of this ", javaType, "\n",
        "\t*\n",
        "\t* @return\tthe value of field data\n",
        "\t*/\n",
        "\tpublic abstract ", getBaseObj(typeMode), " getData();\n",
        "\n\t/**\n", 
        "\t* Compare this ", javaType, "\n",
        "\t* with the input data.\n",
        "\t*\n",
        "\t* @param inputObject\tthe data to be compared with.\n",
        "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
        "\t*/\n",
        "\tpublic abstract boolean equals(Object inputObject);\n",
        "\n\t/**\n", 
        "\t* Print this ", javaType, " to String\n",
        "\t*\n",
        "\t* @return\tcontents of this ", javaType, " in String.\n",
        "\t*/\n",
        "\tpublic abstract String toString();\n",
        "}\n", sep="", collapse="") 

    factoryJType <- memberFactoryName(javaType, names(subklass))     
    memberBody <- mapply(function(jType, thisFactory) {
        defaultAssignStr <- paste("\t\tthis.data = ",
                                getConstructorPart(jType), ";\n", 
                                sep="", collapse="")
        thisBody <- paste("\n/**\n* ", thisFactory, ".java\n", header,
            "\n\npublic class ", thisFactory, " extends ", javaType, " {\n",
            "\tprivate ", jType, " data=null;\n",
            "\tprotected static final String errorMsg = ",
            "\"", thisFactory, " Field data can only be ", jType, ".\";\n",
            "\n\t/**\n", 
            "\t* Empty constructor for ", thisFactory, "\n",
            "\t*/\n",
            "\n\tpublic ", thisFactory, "() {\n",
            defaultAssignStr,
            "\t}\n",
            "\n\t/**\n", 
            "\t* Sets the data value for this ", thisFactory, "\n",
            "\t*\n",
            "\t* @param\tdata\tdata should be an instance of ", jType, "\n",
            "\t*/\n",
            "\tpublic void setData( ", getBaseObj(typeMode), " data) throws Exception {\n",
            "\t\tif( data != null ) {\n",
            "\t\t\ttry{\n",
            "\t\t\t\tthis.data = (", jType, ")data;\n",
            "\t\t\t} catch( Exception ex ) {\n",
            "\t\t\t\tthrow new Exception( ex.getMessage()+errorMsg );\n",
            "\t\t\t}\n",
            "\t\t}\n",
            "\t}\n",
            "\n\t/**\n", 
            "\t* Gets the data value of this ", thisFactory, "\n",
            "\t*\n",
            "\t* @return\tthe value of field data\n",
            "\t*/\n",
            "\tpublic ", getBaseObj(typeMode), " getData() {\n",
            "\t\t return data;\n",
            "\t}\n",
            "\n\t/**\n",
            "\n\t* Print this ", thisFactory, " to String\n",
            "\t*\n",
            "\t* @return\tcontents of this ", thisFactory, " to String\n",
            "\t*/\n",
            "\tpublic String toString() {\n",
            "\t\treturn ", getToStringMethod(jType, "data"), ";\n",
            "\t}\n",
            "\n\t/**\n", 
            "\t* Compare this ", thisFactory, "\n",
            "\t* with the input data.\n",
            "\t*\n",
            "\t* @param inputObject\tthe data to be compared with.\n",
            "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
            "\t*/\n",
            "\tpublic boolean equals(Object inputObject) {\n",
            "\t\tboolean res = getClass().equals(inputObject.getClass());\n",
            "\t\tif(res){\n",
            "\t\t\t", thisFactory, " obj = (", thisFactory, ")inputObject;\n",
            "\t\t\tObject inputData = obj.getData();\n",
            "\t\t\tif((data==null)||(inputData==null))\n",
            "\t\t\t\tres=res&&(data==inputData);\n",
            "\t\t\telse\n",
            "\t\t\t\tres = res&&", getEqualsMethod(jType, "data"),";\n",
            "\t\t}\n",
            "\t\treturn res;\n",
            "\t}\n",
            "}\n", sep="", collapse="")
        thisBody }, subklass, factoryJType)

    factoryBody <- c(unionFactoryBody, unlist(memberBody))
    names(factoryBody) <- c(javaType, factoryJType)
    factoryBody
}

## write the java bean to a file
## param:
##      beanBody: java bean definition in character string
##      javaType: java bean name, short name (without package)
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
## return: NULL
## SIDE EFFECT: create  directory ${pkgName} under current working directory
writeJavaBean <- function( beanBody, javaType, pkgName ) {
        outputPath <- gsub(".", .Platform$file.sep, pkgName, fixed=TRUE)
        dir.create(path=outputPath, showWarnings=FALSE, recursive=TRUE)
        cat(beanBody, file = 
            file.path(outputPath, paste(javaType, ".java", sep="", collapse="")))
        return()
}

## write the java classes to a file
## param:
##      beanBody: a named character vector: name is java class name (short name, 
##          without package), value is the definition of the java classes 
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
## return: NULL
## SIDE EFFECT: create  directory ${pkgName} under current working directory
writeJavaFactory <- function( factoryBodies, pkgName ) {
        outputPath <- gsub(".", .Platform$file.sep, pkgName, fixed=TRUE)
        dir.create(path=outputPath, showWarnings=FALSE, recursive=TRUE)
        mapply(function(contents, name){
            cat(contents, file= 
                file.path(outputPath, paste(name, ".java", sep="", collapse="")))
            }, factoryBodies, names(factoryBodies))
        return()
}

## main function of this section
## param:
##      rType: R S4 class name 
##      jPkgName: java package name, ${mainDir}.${rPkgName}.data
##      slotListJ: a named vector where names are slot names of the R S4 class, 
##                  and values are the corresponding java data types for the slots
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: java bean name, shot name (without package name in front)
generateJavaBean <- function( rType, jPkgName, slotListJ, typeMode ) {
    javaType <- capName(mangleName(rType))
    classDoc <- getClassDoc(rType)
    beanBody <- createJavaBean(javaType, jPkgName, slotListJ, classDoc, typeMode)
    writeJavaBean( beanBody, javaType, jPkgName ) 
    javaType
} 

## main function of this section
## param:
##      rType: R S4 class name 
##      jPkgName: java package name, ${mainDir}.${rPkgName}.data
##      subklassJ: a named vector where names are member class names of the R ClassUnion, 
##                  and values are the corresponding java data types for them
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: java bean name, shot name (without package name in front)
generateUnionFactory <- function( rType, jPkgName, subklassJ, typeMode ) {
    javaType <- paste(capName(mangleName(rType)), "Factory", sep="", collapse="")
    factoryBodies <- createUnionFactory(javaType, jPkgName, subklassJ, typeMode)
    writeJavaFactory(factoryBodies, jPkgName)
    javaType 
}

   
